package org.jboss.security.jce;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import java.security.Provider;
import java.security.Security;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import org.jboss.security.jce.JCEProviderServiceMBean;
import org.jboss.system.ServiceMBeanSupport;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class JCEProviderService extends ServiceMBeanSupport implements JCEProviderServiceMBean {
  private ArrayList addedProviders = new ArrayList();
  
  private Element providers;
  
  public void setJceProviders(Element element) {
    this.providers = element;
    if (getState() != 7)
      synchronized (org.jboss.security.jce.JCEProviderService.class) {
        removeProviders();
        loadProviders();
      }  
  }
  
  public Element getJceProviders() {
    return this.providers;
  }
  
  protected void createService() throws Exception {
    synchronized (org.jboss.security.jce.JCEProviderService.class) {
      loadProviders();
    } 
  }
  
  protected void destroyService() {
    synchronized (org.jboss.security.jce.JCEProviderService.class) {
      removeProviders();
    } 
  }
  
  private void loadProviders() {
    int n = 0;
    if (this.providers != null) {
      this.addedProviders = new ArrayList();
      NodeList reqdProviders = this.providers.getElementsByTagName("provider");
      n = reqdProviders.getLength();
      int i = 0;
      while (true) {
        if (i < n) {
          Node reqdProvider = reqdProviders.item(i);
          int requestedPosition = 0;
          if (1 == reqdProvider.getNodeType()) {
            Provider provider;
            String providerClass;
            int pos;
            Element prov = (Element)reqdProvider;
            if (prov.hasAttribute("class")) {
              providerClass = prov.getAttribute("class");
            } else {
              this.log.warn("A provider element must, at the very least, have a class attribute: " + prov);
              i++;
            } 
            try {
              provider = (Provider)Class.forName(providerClass).newInstance();
            } catch (InstantiationException e1) {
              this.log.warn("Unable to instantiate an instance of the JCE Provider class " + providerClass, e1);
            } catch (IllegalAccessException e1) {
              this.log.warn("No permission to access the JCE Provider class " + providerClass, e1);
            } catch (ClassNotFoundException e1) {
              this.log.warn("Could not find the JCE Provider class " + providerClass, e1);
            } catch (ClassCastException e1) {
              this.log.warn("The Class " + providerClass + " is not a java.security.Provider");
            } 
            String providerName = provider.getName();
            if (prov.hasAttribute("position"))
              try {
                requestedPosition = Integer.parseInt(prov.getAttribute("position"));
              } catch (NumberFormatException e) {
                this.log.warn("the position '" + prov.getAttribute("position") + "' is not a valid number. This provider has to go to the end of the line. " + prov);
              }  
            if (requestedPosition < 1) {
              pos = Security.addProvider(provider);
            } else {
              pos = Security.insertProviderAt(provider, requestedPosition);
            } 
            if (pos == -1) {
              int exPos = Arrays.<Provider>asList(Security.getProviders()).indexOf(provider);
              this.log.info("The provider " + providerName + " already exists at position " + exPos);
            } else if (requestedPosition >= 1 && pos != requestedPosition) {
              this.log.info("The position " + requestedPosition + " was requested for Provider " + providerName + " but it was added at position " + pos);
              this.addedProviders.add(providerName);
            } else {
              this.log.info("The Provider " + providerName + " was added at position " + pos);
              this.addedProviders.add(providerName);
            } 
          } else if (this.log.isDebugEnabled()) {
            this.log.debug("Ignoring node" + reqdProvider);
          } 
        } else {
          break;
        } 
        i++;
      } 
      if (this.addedProviders.size() == 1) {
        this.log.info(this.addedProviders.size() + " JCE Provider was actually loaded.");
      } else {
        this.log.info(this.addedProviders.size() + " JCE Providers were actually loaded.");
      } 
    } 
    if (n < 1)
      this.log.info("No JCE Providers were requested."); 
  }
  
  private void removeProviders() {
    for (Iterator iter = this.addedProviders.iterator(); iter.hasNext(); ) {
      String providerName = iter.next();
      try {
        Security.removeProvider(providerName);
      } catch (Exception e) {
        this.log.warn("Failed to remove Provider " + providerName);
      } 
    } 
  }
}
